'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      Program: function Program(programPath) {
        var store = new _Store2.default();

        programPath.traverse({
          AssignmentExpression: function AssignmentExpression(path) {
            var _path$node = path.node,
                left = _path$node.left,
                right = _path$node.right;
            var object = left.object,
                property = left.property;
            var identifier = object.name;


            if ((0, _assertions.isHandledAssignment)(left, right, property)) {
              var elements = right.elements;


              elements.forEach(function (element) {
                return store.add(identifier, element.value);
              });
              path.remove();

              return;
            }

            if ((0, _assertions.isPropsAssignment)(left, right, property)) {
              var properties = right.properties;

              properties.forEach(function (item) {
                return store.add(identifier, item.key.name);
              });
            }
          },
          ClassProperty: function ClassProperty(path) {
            var _path$node2 = path.node,
                key = _path$node2.key,
                value = _path$node2.value;


            if ((0, _assertions.isHandledProperty)(key, value, path.node.static)) {
              var elements = value.elements;


              elements.forEach(function (element) {
                return store.add((0, _helpers.findClassIdentifier)(path), element.value);
              });
              path.remove();

              return;
            }

            if ((0, _assertions.isPropsProperty)(key, value, path.node.static)) {
              var properties = value.properties;

              properties.forEach(function (property) {
                return store.add((0, _helpers.findClassIdentifier)(path), property.key.name);
              });
            }
          }
        });

        programPath.traverse({
          'ClassDeclaration|FunctionDeclaration': function ClassDeclarationFunctionDeclaration(path) {
            var name = path.node.id.name;


            if (!store.has(name)) return;
            if (t.isExportDeclaration(path.parentPath)) path = path.parentPath;

            path.insertAfter((0, _helpers.generateExpression)(store, name));
          }
        });
      }
    }
  };
};

var _assertions = require('./assertions');

var _helpers = require('./helpers');

var _Store = require('./Store');

var _Store2 = _interopRequireDefault(_Store);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }